#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include "serialport.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

extern char *BAUDRATE[18];

bool setimage(GtkWidget *widget, const char *filename);
bool fill_combobox(GtkWidget *widget, int n, char **text);
gboolean incomming_data(GIOChannel *source, GIOCondition condition, gpointer data);
gint setstatus(gpointer data);


void on_window1_destroy(GtkObject *object, gpointer user_data)
{
	g_print("on_window1_destroy\n");
	gtk_main_quit();
}


void on_oeffnen_clicked(GtkButton *button, gpointer user_data)
{
    g_print("on_oeffnen_clicked\n");
	GIOChannel *channel; 
	guint timer;
	
	/*An das Hauptfenster gebundenen Daten holen*/
	struct serial_cfg *sr_cfg = gtk_object_get_data(user_data,"sr_cfg"); 
	GtkEntry *schnittstelle = gtk_object_get_data(user_data,"schnittstelle");
	GtkEntry *baudrate = gtk_object_get_data(user_data,"baudrate");
	GtkWidget *schliessen = gtk_object_get_data(user_data,"schliessen");
	
	char *dev = gtk_entry_get_text(schnittstelle);
	int baud  = atoi(gtk_entry_get_text(baudrate));
	
	if(sr_cfg->fd == -1)
	{	
		if(serial_init(dev, baud, sr_cfg) != -1)
		{
			g_print("schnittstelle %s geoeffnet\n",dev);
			/*GIOChannel einrichten*/
			channel =  g_io_channel_unix_new(sr_cfg->fd);
			g_io_add_watch(channel,G_IO_IN,incomming_data, user_data);
			/*und an das Hauptfenster binden*/
			gtk_object_set_data(GTK_OBJECT(user_data),"channel",channel);
			/*Funktion setstatus wird 1 mal/s aufgerufen 
			  und ueberprueft Steuerleitungen*/
			timer = gtk_timeout_add(1000, setstatus, user_data);
			gtk_object_set_data(GTK_OBJECT(user_data),"timer",(gpointer)timer);
			/*Schalter Oeffnen/Schliessen aktivieren bzw. deaktivieren*/
			gtk_widget_set_sensitive(GTK_WIDGET(schliessen), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
		}
		else	
			g_print("%s konnte nicht geoeffnet werden!\n",dev);
	}
	else
	{
		g_print("Schnitstelle ist schon geöffnet worden fd != -1 !!!\n");
	}			
}




void on_schliessen_clicked(GtkButton *button, gpointer user_data)
{
	g_print("on_schliessen_clicked\n");

	/*An das Hauptfenster gebundenen Daten holen*/
	struct serial_cfg *sr_cfg = gtk_object_get_data(user_data,"sr_cfg"); 
	GtkWidget *oeffnen = gtk_object_get_data(user_data,"oeffnen");
	GIOChannel *channel = gtk_object_get_data(user_data,"channel");
	guint timer = (guint)gtk_object_get_data(user_data,"timer");

	if(sr_cfg->fd > -1)
	{	
		/*Schnitstelle freigeben, geht auch mit close(sr_cfg->fd)*/
		serial_relase(sr_cfg);
		sr_cfg->fd = -1;
		/*GIOChannel entfernen*/
		g_io_channel_unref(channel);
		gtk_object_remove_data(user_data,"channel");
		/*Timeout fuer die funktion setstatus entfernen*/
		gtk_timeout_remove(timer);
		gtk_object_remove_data(user_data,"timer");
		/*Schalter Oeffnen/Schliessen aktivieren bzw. deaktivieren*/
		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(oeffnen), TRUE);
		g_print("Schnitstelle geschlossen\n");

	}
	else
	{
		g_print("Schnitstelle ist nicht offen fd == -1 !!!\n");
	}
}

void on_senden_changed(GtkEditable *editable, gpointer user_data)
{
	g_print("on_senden_changed\n");

	
}

void on_dtr_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	g_print("on_dtr_toggled\n");
	struct serial_cfg *sr_cfg = gtk_object_get_data(user_data,"sr_cfg");
	
	if(gtk_toggle_button_get_active(togglebutton))
	{
		set_dtr(true, sr_cfg);				
		setimage(gtk_object_get_data(user_data,"dtr_stat"), "./terminal_on.xpm");	
	}
	else
	{	
		set_dtr(false, sr_cfg);		
		setimage(gtk_object_get_data(user_data,"dtr_stat"), "./terminal_off.xpm");
	}
}

void on_rts_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	g_print("on_rts_toggled\n");
	struct serial_cfg *sr_cfg = gtk_object_get_data(user_data,"sr_cfg");
	
	if(gtk_toggle_button_get_active(togglebutton))
	{	
		set_rts(true, sr_cfg);
		setimage(gtk_object_get_data(user_data,"rts_stat"), "./terminal_on.xpm");
	}
	else
	{	
		set_rts(false, sr_cfg);	
		setimage(gtk_object_get_data(user_data,"rts_stat"), "./terminal_off.xpm");
	}		
}

void on_senden_bu_clicked(GtkButton *button, gpointer user_data)
{
	g_print("on_senden_bu_clicked\n");
	int size[2];
	gchar *buffer;
	/*An das Hauptfenster gebundenen Daten holen*/	
	struct serial_cfg *sr_cfg = gtk_object_get_data(user_data,"sr_cfg"); 
	GtkText *senden = gtk_object_get_data(user_data,"senden");
	GtkText *gesendet = gtk_object_get_data(user_data,"gesendet");	

	if((size[0] = gtk_text_get_length(senden)) > 0)
	{
		buffer = gtk_editable_get_chars(GTK_EDITABLE(senden), 0, size[0]);
		size[1] = gtk_text_get_length(gesendet);
		gtk_editable_insert_text(GTK_EDITABLE(gesendet),buffer,size[0],&size[1]);
		gtk_editable_delete_text(GTK_EDITABLE(senden),0,size[0]);
		g_print("%s\n",buffer);
		/*Daten senden, hier kann auch die Funktion 
		  g_io_channel_write verwendet werden*/		
		send_data((unsigned char*)buffer, size[0], sr_cfg);
		g_free(buffer);/*wegen gtk_editable_get_chars*/
	}
	else
			g_print("nichts zu senden!!\n");		
}
